﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using PostSharp.Aspects;

namespace ContactManager.Aspects
{
    [Serializable]
    class TracePerformanceAttribute : OnMethodBoundaryAspect
    {
        private static Stopwatch stopwatch = Stopwatch.StartNew();
        private string methodName;

        public override void CompileTimeInitialize(System.Reflection.MethodBase method, AspectInfo aspectInfo)
        {
            this.methodName = method.DeclaringType.FullName +
                              "." + method.Name;
        }
        public override void OnEntry(MethodExecutionArgs args)
        {
            args.MethodExecutionTag = stopwatch.ElapsedTicks;

            Trace.TraceInformation("Entering method {0}.",
               this.methodName);

            Trace.Indent();
        }

        public override void OnExit(MethodExecutionArgs args)
        {
            long before = (long) args.MethodExecutionTag;

            double ms = 1000.0*(stopwatch.ElapsedTicks - before)
                        /Stopwatch.Frequency;

            Trace.Unindent();
            Trace.TraceInformation("Method {0} took {1} ms.",
                this.methodName, ms);
        }
    }
}
